
------------------------------------------------------------------------
--
-- ShieldFrames
--
-- Copyright (C) 2010  Philipp Schmidt
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to:
--
-- Free Software Foundation, Inc.,
-- 51 Franklin Street, Fifth Floor,
-- Boston, MA  02110-1301, USA.
--
--
------------------------------------------------------------------------


local ShieldFrames = _G.ShieldFrames;

local EffectProto = {};
ShieldFrames.EffectProto = EffectProto;

local EffectCache = {};


local function OnUpdateDuration(self)
	local t = GetTime();
	
	if(t > self.timeout) then
		self:SetValue(0);
		self:SetScript("OnUpdate", nil);
	else
		self:SetValue(self.timeout - t);
	end
end

function EffectProto:New(profile)
	if(#EffectCache > 0) then
		local cachedEffect = tremove(EffectCache, 1);
		
		cachedEffect:ApplyProfile();
		
		return cachedEffect;
	end

	local o = setmetatable({}, self);
	self.__index = self;
	
	o.profile = profile;
	
	o:CreateFrame();
	
	return o;
end

function EffectProto:Release()
	self.frame:Hide();

	self.frame:SetScript("OnUpdate", nil);	
	self.amount = nil;
	
	tinsert(EffectCache, self);
end

function EffectProto:CreateFrame()
	local frame = CreateFrame("Frame", nil, UIParent);
	self.frame = frame;
	
	frame:Hide();
			
	local icon = frame:CreateTexture(nil, "ARTWORK");
	self.icon = icon;
	
	icon:SetPoint("TOPLEFT", frame);
	icon:SetPoint("BOTTOMLEFT", frame);
	icon:SetTexCoord(0.07, 0.93, 0.07, 0.93);
	
	local amountBar = CreateFrame("StatusBar", nil, frame);
	self.amountBar = amountBar;
	
	amountBar:SetStatusBarTexture([[Interface\Addons\AbsorbsViewer\BantoBar]]);
	amountBar:SetPoint("TOPLEFT", icon, "TOPRIGHT");
	amountBar:SetPoint("TOPRIGHT", frame);
	amountBar:SetOrientation("HORIZONTAL");
	
	local amountBg = amountBar:CreateTexture(nil, "BACKGROUND");
	self.amountBg = amountBg;
	
	amountBg:SetTexture([[Interface\Addons\AbsorbsViewer\BantoBar]]);
	amountBg:SetVertexColor(0.6, 0.6, 0.6, 0.8);
	amountBg:SetAllPoints();
	
	local leftText = amountBar:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmallOutline");
	self.leftText = leftText;
	
	leftText:SetPoint("LEFT", amountBar, "LEFT", 2, 0);
	
	local rightText = amountBar:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmallOutline");
	self.rightText = rightText;

	rightText:SetPoint("RIGHT", amountBar, "RIGHT", -2, 0);
	
	local durationBar = CreateFrame("StatusBar", nil, frame);
	self.durationBar = durationBar;
	
	durationBar:SetStatusBarTexture([[Interface\Addons\Recount\textures\statusbar\Flat]]);
	durationBar:SetStatusBarColor(0, 0, 1, 1.0);
	durationBar:SetPoint("BOTTOMLEFT", icon, "BOTTOMRIGHT");
	durationBar:SetPoint("BOTTOMRIGHT", frame);
	durationBar:SetOrientation("HORIZONTAL");
	
	amountBar:SetPoint("BOTTOMLEFT", durationBar, "TOPLEFT", 0, 1);
	amountBar:SetPoint("BOTTOMRIGHT", durationBar, "TOPRIGHT", 0, 1);
	
	local durationBg = durationBar:CreateTexture(nil, "BACKGROUND");
	self.durationBg = durationBg;
	
	durationBg:SetTexture([[Interface\Addons\Recount\textures\statusbar\Flat]]);
	durationBg:SetVertexColor(0.6, 0.6, 0.6, 0.8);
	durationBg:SetAllPoints();
	
	self:ApplyProfile();
end

function EffectProto:ApplyProfile()
	local profile = self.profile;

	self.frame:SetScale(profile.scale);		
	self.frame:SetWidth(profile.width);
	self.frame:SetHeight(profile.effectHeight);
	self.frame:SetAlpha(profile.effectAlpha);
	
	self.icon:SetWidth(profile.effectHeight);
	self.icon:SetHeight(profile.effectHeight);
	
	self.leftText:SetTextHeight(profile.textHeight);	
	self.rightText:SetTextHeight(profile.textHeight);		
	
	self.durationBar:SetHeight(profile.durationHeight);

	if(self.amount) then
		self:UpdateValues(self.amount);
	end
end

function EffectProto:SetupValues(sourceName, destName, spellId, amount, quality, duration)
	local spellName, _, spellIcon = GetSpellInfo(spellId);	
	
	self.icon:SetTexture(spellIcon);
				
	self.amountBar:SetMinMaxValues(0, amount);
	self.max = amount;
			
	self.sourceName = sourceName;
	self.destName = destName;
	self.spellName = spellName;
	
	self:UpdateValues(amount, quality, duration);
end

function EffectProto:UpdateValues(amount, quality, duration)
	self.amount = amount;

	if(amount > self.max) then
		self.max = amount;
		self.amountBar:SetMinMaxValues(0, amount);
		
		-- ask for repositioning maybe
	end
	
	if(quality) then
		self.amountBar:SetStatusBarColor(1 - quality, quality, 0, 1.0);
	end	
	
	if(duration) then
		self.durationBar:SetScript("OnUpdate", nil);
		self.durationBar:SetMinMaxValues(0, duration);
		self.durationBar:SetValue(duration);	
		self.durationBar.duration = duration;
		self.durationBar.timeout = GetTime() + duration;
		self.durationBar:SetScript("OnUpdate", OnUpdateDuration);
	end
				
	self.amountBar:SetValue(amount);		
	
	local str = self.profile.textFormat	
	
	str = gsub(str, "%%s", self.sourceName);
	str = gsub(str, "%%d", self.destName);
	str = gsub(str, "%%n", self.spellName);
	str = gsub(str, "%%a", self.amount);
	str = gsub(str, "%%m", self.max);
	str = gsub(str, "%%p", (self.amount / self.max) * 100);
	
	local leftStr, rightStr = strsplit("|", str);
	
	self.leftText:SetText(leftStr);
	
	if(rightStr) then
		self.rightText:SetText(rightStr);	
	end
end

-- alignEffect is NOT guaranteed to be an EffectProto object, but only to have a frame property
function EffectProto:AlignTo(alignEffect)
	local profile = self.profile;
	
	self.frame:ClearAllPoints();

	if(profile.growthDirection == -1) then
		self.frame:SetPoint("TOPLEFT", alignEffect.frame, "BOTTOMLEFT", 0, -1 * profile.effectSpacing);
	else
		self.frame:SetPoint("BOTTOMLEFT", alignEffect.frame, "TOPLEFT", 0, profile.effectSpacing);
	end
end
